const path = require('path');
const log = require('./log');

const kinds = {};

module.exports.genImportDeclare = (names, tsFilePath, autoMock) => {
  const mws = {};
  const npmImports = [];
  const proImports = [];
  names.forEach((name) => {
    if (!kinds[name]) {
      return;
    }
    let pipes = [];
    pipes = pipes.concat(kinds[name]);
    if (autoMock) {
      pipes = pipes.concat(kinds._mock || []);
    }
    pipes.forEach((p) => {
      if (mws[p.name]) return;
      // relate path or npm path
      if (p.path.startsWith('.')) {
        const pathOfPrj = path.join(process.cwd(), 'src', path.dirname(p.path));
        const relativePath2File = path.relative(path.dirname(tsFilePath), pathOfPrj) || './';
        mws[p.name] = path.join(relativePath2File, path.basename(p.path));
        proImports.push(`import ${p.name}_kind from '${mws[p.name]}';`);
      } else {
        mws[p.name] = p.path;
        npmImports.push(`import ${p.name}_kind from '${mws[p.name]}';`);
      }
    });
  });
  return { imports: [].concat(npmImports, proImports), mws: Object.keys(mws) };
};

const setKinds = (kindFiles) => {
  // eslint-disable-next-line no-restricted-syntax
  for (const customSol of kindFiles) {
    try {
      // eslint-disable-next-line import/no-dynamic-require
      let userDefined = require(customSol);
      userDefined = userDefined.default || userDefined;
      Object.keys(userDefined).forEach(k => {
        if (!userDefined[k]) return;
        if (k === '[defaultKind]') {
          module.exports.defaultKind = userDefined[k];
          return
        }
        if (typeof(userDefined[k]) === 'string') {
          kinds[k] = [{ name: path.basename(k), path: userDefined[k] }];
        } else {
          kinds[k] = [].concat(userDefined[k])
        }
      });
      log(`已加载kind: ${customSol}`);
    } catch (e) {
      log(`未加载${customSol}`);
    }
  }
};

module.exports.setKinds = setKinds;
module.exports.kinds = kinds;
module.exports.defaultKind = 'ajax';
