const _ = require('lodash');
const statementsType = require('../core/statement');

const generator = (ast) => {
  let defaultExport;
  const keyStatement = {};
  ast.statements.forEach((st) => {
    if (st instanceof statementsType.Clazz && st.modifiers.indexOf('default') !== -1) {
      defaultExport = st;
      return;
    }
    keyStatement[st.name] = st;
  });
  if (!defaultExport) throw new Error('cant\'t find default export');
  if (!_.get(defaultExport, 'heritage.typeArguments')) return {};
  const props = defaultExport.heritage.typeArguments[0];
  const output = stepInProps(props, keyStatement)[props.name];
  delete output.$isRequired;
  delete output.$tip;
  delete output.$type;
  return output;
};

const stepInProps = (struct, keyStatement) => {
  if (!struct) {
    return
  }
  const json = {
    $isRequired: !struct.isOptional,
    $tip: struct.comment,
    ...struct.config,
  };
  if (struct.type instanceof statementsType.UnionType) {
    json['$type'] = 'enum';
    json['$options'] = struct.type.map((t) => {
      return _.get(t, 'type.name', _.get(t, 'type.text', t.type));
    });
  } else if (struct instanceof statementsType.TypeAlias || struct instanceof statementsType.Interface) {
    if (struct.isSimple && struct.type) {
      json['$type'] = struct.type.name || struct.type;
    } else if (struct.type && Object.keys(struct.type).length === 0) {
      json['$type'] = 'Object';
    } else {
      json['$type'] = 'nest';
      struct.forEach((subStruct) => {
        json[subStruct.name] = stepInProps(subStruct, keyStatement);
      });
    }
  } else if (struct instanceof statementsType.TypeReference) {
    json['$type'] = 'nest';
    // refer from other file
    if (!keyStatement[struct.name]) {
      return json;
    }
    json[struct.name] = stepInProps(keyStatement[struct.name], keyStatement);
  } else if (struct instanceof statementsType.Func) {
    json['$type'] = 'func';
    if (!json.initialValue) {
      json.initialValue = `(${struct.parameters.map(p => p.name).join(', ')}) => {}`;
    }
  } else if (struct.isSimple && struct.type instanceof String) {
    json[struct.name] = struct.type.name;
    json['$type'] = struct.type.name;
  }
  return json;
};

module.exports = generator;
